Attribute VB_Name = "DDExt"
Option Explicit

Public Done As Boolean

Public DD As DirectDraw7
Public Device As Direct3DDevice7
Public DX7 As DirectX7
Public IMode As Direct3D7
Public Clipper() As DirectDrawClipper

Public sPrimary As DirectDrawSurface7
Public sBackBuffer As DirectDrawSurface7
Public s3D As DirectDrawSurface7
Public sZbuffer As DirectDrawSurface7

Public m_OgreView As D3DVIEWPORT7
Public DD_Guid As String
Public DD_GuidDesc As String
Public D3D_Guid As String
Public D3D_GuidDesc As String
Public winBPP As Long

Public Surfaces As OGRESurfaces
Public Lights As OGRELights
Public Camera As OGRECams
Public Meshes As OGREMeshes
Public TDS As OGRE3DSmodule
Public GUIelements As OGREGUI

Public ThreeDim As Boolean          ' Are we using d3dim mode?
Public InitTrue As Boolean          ' For the driver selection. If the user has completed a driver selection or not
Public fScreen As Boolean           ' Is this a fullscreen app?
Public ResizeOK As Boolean          ' Wether or not the first instance of a win app is ok to resize

Public WindowCount As Byte          ' Number of windows, for windowed mode, that need clipper routines
Public WindowHwnd() As Long        ' The handle to the windows hwnds
Public SubClassHwnd As Long      ' The main window for keyboard handling/resizing/etc...
Public PrevWndProc As Long

Public matSet As D3DMATRIX
Public ClipWidth As Single
Public ClipHeight As Single

Public m_WinRect As RECT
Public FPSON As Boolean
Public CameraInfo As Boolean
Public PolyCount As Long
Public oldTexture As String

Public Keys(255) As Boolean
Public anErr As New AppErrs

' For subclassing the OGRE Form
Private Declare Function SetWindowLong& Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long)
Public Const GWL_WNDPROC = (-4&)

Public Function ReInit() As Boolean
    On Error Resume Next
    
    Set Device = Nothing
    Set sZbuffer = Nothing
    Set s3D = Nothing
    Set sBackBuffer = Nothing
    Set sPrimary = Nothing
    DX7.GetWindowRect WindowHwnd(1), m_WinRect
    ReInit = LoadDevices
    
    Camera.LoadView
    Dim tLight As vLight
    For Each tLight In Lights.AllLights
        If tLight.Enabled Then Lights.Enable tLight.Name
    Next
    Set tLight = Nothing
    oldTexture = ""
    GUIelements.GetElement("CONSOLE").Resize
End Function

Public Function LoadDevices() As Boolean
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "MAINDD_Init: LoadDevices"
    
    ResizeOK = False
    Dim I As Long
        
    If fScreen Then
        ShowCursor False
        DD.SetCooperativeLevel WindowHwnd(1), DDSCL_EXCLUSIVE Or DDSCL_FULLSCREEN Or DDSCL_ALLOWREBOOT
        DD.SetDisplayMode m_WinRect.Right, m_WinRect.Bottom, winBPP, 0, 0
    Else
        DD.SetCooperativeLevel WindowHwnd(1), DDSCL_NORMAL
        If WindowHwnd(1) = OGREForm.hWnd Then OGREForm.Show
    End If
    
    Dim ddsd As DDSURFACEDESC2
    ErrString = "MAINDD_Init: Primary Surface"
    With ddsd
        .lFlags = DDSD_CAPS
        .ddsCaps.lCaps = DDSCAPS_PRIMARYSURFACE
        If fScreen Then
            .lFlags = .lFlags Or DDSD_BACKBUFFERCOUNT
            .ddsCaps.lCaps = .ddsCaps.lCaps Or DDSCAPS_COMPLEX Or DDSCAPS_FLIP Or DDSCAPS_VIDEOMEMORY
            .lBackBufferCount = 1
        End If
        Set sPrimary = DD.CreateSurface(ddsd)
        
        ErrString = "MAINDD_Init: Backbuffer"
        .lFlags = DDSD_CAPS
        If fScreen Then
            .ddsCaps.lCaps = DDSCAPS_BACKBUFFER
            Set sBackBuffer = sPrimary.GetAttachedSurface(.ddsCaps)
        Else
            .lFlags = .lFlags Or DDSD_HEIGHT Or DDSD_WIDTH
            .ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN Or DDSCAPS_VIDEOMEMORY Or DDSCAPS_3DDEVICE
            .lWidth = m_WinRect.Right - m_WinRect.Left
            .lHeight = m_WinRect.Bottom - m_WinRect.Top
            Set sBackBuffer = DD.CreateSurface(ddsd)
            
            ErrString = "MAINDD_Init: Clipper"
            Set Clipper(1) = DD.CreateClipper(0)
        End If
        Dim a As New StdFont
        a.Size = 10
        a.Name = "Arial"
        sBackBuffer.SetFont a
        Set a = Nothing
    End With
    
    If ThreeDim Then
        With ddsd
            ErrString = "MAINDD_Init: Z-Buffer"
            Dim DDSD2 As DDSURFACEDESC2
            Dim fenum As Direct3DEnumPixelFormats
            Dim ddpfZBuffer As DDPIXELFORMAT
            Set fenum = IMode.GetEnumZBufferFormats(D3D_Guid)
            For I = 1 To fenum.GetCount()
                Call fenum.GetItem(I, ddpfZBuffer)
                If ddpfZBuffer.lFlags = DDPF_ZBUFFER Then
                    With DDSD2
                        .lFlags = DDSD_CAPS Or DDSD_WIDTH Or DDSD_HEIGHT Or DDSD_PIXELFORMAT
                        .ddsCaps.lCaps = .ddsCaps.lCaps Or DDSCAPS_ZBUFFER
                    .lWidth = m_WinRect.Right - m_WinRect.Left
                    .lHeight = m_WinRect.Bottom - m_WinRect.Top
                    .ddpfPixelFormat = ddpfZBuffer
                    End With
                    Set sZbuffer = DD.CreateSurface(DDSD2)
                    Exit For
                End If
            Next
            Set fenum = Nothing
        
            ErrString = "MAINDD_Init3D: Z-Buffer. No ZBuffer format found"
            If sZbuffer Is Nothing Then GoTo errs
        
            ErrString = "MAINDD_Init3D: 3D Surface"
            If fScreen Then
                .lFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
                .ddsCaps.lCaps = DDSCAPS_3DDEVICE
                .lWidth = m_WinRect.Right - m_WinRect.Left
                .lHeight = m_WinRect.Bottom - m_WinRect.Top
                Set s3D = DD.CreateSurface(ddsd)
                s3D.AddAttachedSurface sZbuffer
                Set Device = IMode.CreateDevice(D3D_Guid, s3D)
            Else
                sBackBuffer.AddAttachedSurface sZbuffer
                Set Device = IMode.CreateDevice(D3D_Guid, sBackBuffer)
            End If
        End With
        
        ErrString = "MAINDD_Init3D: Viewport"
        With m_OgreView
            .lWidth = m_WinRect.Right - m_WinRect.Left
            .lHeight = m_WinRect.Bottom - m_WinRect.Top
            .MinZ = 0#
            .MaxZ = 1#
        End With
        Device.SetViewport m_OgreView
    
        ErrString = "MAINDD_Init3D: Device States"
        Dim mtrl As D3DMATERIAL7
        mtrl.Ambient.R = 0.75: mtrl.Ambient.g = 0.75: mtrl.Ambient.b = 0.75
        With Device
            .SetMaterial mtrl
            .SetRenderState D3DRENDERSTATE_AMBIENT, DX7.CreateColorRGBA(0.7, 0.7, 0.7, 1#)
            .SetTextureStageState 0, D3DTSS_COLOROP, D3DTOP_MODULATE
        End With
    Else
        Set Device = IMode.CreateDevice(D3D_Guid, sBackBuffer)
    End If
    
    ErrString = "MAINDD_Init: End Conditions"
    
    If GUIelements Is Nothing Then
        Set GUIelements = New OGREGUI
        GUIelements.AddElement Console, "CONSOLE"
        With GUIelements.GetElement("CONSOLE")
            .Prompt = "# "
            .TextColor = RGB(192, 196, 200)
        End With
    End If
    
    ResizeOK = True
    sPrimary.SetForeColor RGB(192, 196, 200)
    LoadDevices = True
    Exit Function
errs:
    Call anErr.AppErr(ErrString)
End Function

Public Sub QuitDX()
    On Error Resume Next
    Dim Index As Byte
    For Index = 1 To WindowCount
        SetWindowLong WindowHwnd(Index), GWL_WNDPROC, PrevWndProc
    Next Index
    
    If Not (DD Is Nothing) Then
        DD.RestoreDisplayMode
        DD.SetCooperativeLevel WindowHwnd(1), DDSCL_NORMAL
    End If
    
    Set GUIelements = Nothing
    Set TDS = Nothing
    Set Surfaces = Nothing
    Set Camera = Nothing
    Set Lights = Nothing
    Set Meshes = Nothing
    Set sZbuffer = Nothing
    Set s3D = Nothing
    Set sBackBuffer = Nothing
    Set sPrimary = Nothing
    For Index = 1 To WindowCount
        Set Clipper(Index) = Nothing
    Next Index
    Set Device = Nothing
    Set IMode = Nothing
    Set DD = Nothing
    Set DX7 = Nothing
    Set anErr = Nothing
    ShowCursor True
End Sub

