VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "OGRELights"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

Private m_lights As Collection
Private LightsIndex As Long

Public Property Get AllLights() As Collection
    Set AllLights = m_lights
End Property

Public Property Get Info(Lightname As String) As vLight
    On Error Resume Next
    
    If m_lights Is Nothing Then
        GUIelements.GetElement("CONSOLE").PrintData "Error: No lights initialized."
    ElseIf m_lights(Lightname) Is Nothing Then
        GUIelements.GetElement("CONSOLE").PrintData "Error: Requested light " & Lightname & " is not initialized."
    Else
        Set Info = m_lights(Lightname)
    End If
End Property

Public Sub Delete(Lightname As String)
    If Info(Lightname).Enabled Then Enable Lightname, False
    m_lights.Remove Lightname
End Sub

Public Sub CreatePoint(Lightname As String, Rc As Single, Gc As Single, Bc As Single, _
    Xpos As Single, Ypos As Single, Zpos As Single, Optional nRange As Single = 100)
    
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: CreatePoint"
    
    Dim C As D3DCOLORVALUE
    With C
        .a = 1
        .R = Rc
        .g = Gc
        .b = Bc
    End With
        
    LightsIndex = LightsIndex + 1
    Dim tLight As New vLight
    With tLight
        .Ambient = C: .Diffuse = C: .Specular = C
        .Position = MakeVector(Xpos, Ypos, Zpos)
        .lType = D3DLIGHT_POINT
        .Range = nRange
        .Attenuation0 = 1
        .LightIndex = LightsIndex
        .Name = Lightname
    End With
    
    If m_lights Is Nothing Then Set m_lights = New Collection
    m_lights.Add tLight, Lightname
    Enable Lightname
    Set tLight = Nothing
        
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Public Sub Move(Lightname As String, Optional Xpos As Single, Optional Ypos As Single, Optional Zpos As Single)
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: Move"
        
    With Info(Lightname)
        .Position = MakeVector(IIf(IsMissing(Xpos), .Position.x, Xpos), _
                               IIf(IsMissing(Ypos), .Position.y, Ypos), _
                               IIf(IsMissing(Zpos), .Position.z, Zpos))
        Enable Lightname, .Enabled
    End With
    
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Public Sub Range(Lightname As String, nRange As Single)
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: Range"
            
    With Info(Lightname)
        If nRange >= 0 Then .Range = nRange
        Enable Lightname, .Enabled
    End With
    
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Public Sub Ambient(Lightname As String, Optional Ra As Single, Optional Ga As Single, Optional Ba As Single)
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: Ambient"
            
    Dim C As D3DCOLORVALUE
    With Info(Lightname)
        C.R = IIf(IsMissing(Ra), .Ambient.R, Ra)
        C.g = IIf(IsMissing(Ga), .Ambient.g, Ga)
        C.b = IIf(IsMissing(Ba), .Ambient.b, Ba)
        C.a = 1
        .Ambient = C
        Enable Lightname, .Enabled
    End With
    
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Public Sub Diffuse(Lightname As String, Optional Rd As Single, Optional Gd As Single, Optional Bd As Single)
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: Diffuse"
        
    Dim C As D3DCOLORVALUE
    With Info(Lightname)
        C.R = IIf(IsMissing(Rd), .Diffuse.R, Rd)
        C.g = IIf(IsMissing(Gd), .Diffuse.g, Gd)
        C.b = IIf(IsMissing(Bd), .Diffuse.b, Bd)
        C.a = 1
        .Ambient = C
        Enable Lightname, .Enabled
    End With
    
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Public Sub Specular(Lightname As String, Optional Rs As Single, Optional Gs As Single, Optional Bs As Single)
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: Specular"
        
    Dim C As D3DCOLORVALUE
    With Info(Lightname)
        C.R = IIf(IsMissing(Rs), .Specular.R, Rs)
        C.g = IIf(IsMissing(Gs), .Specular.g, Gs)
        C.b = IIf(IsMissing(Bs), .Specular.b, Bs)
        C.a = 1
        .Ambient = C
        Enable Lightname, .Enabled
    End With
    
    
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Public Sub Enable(Lightname As String, Optional TurnOn As Boolean = True)
    On Error GoTo errs
    Dim ErrString As String
    ErrString = "OGRELight: Enable"
        
    With Info(Lightname)
        If TurnOn Then
            Dim theLight As D3DLIGHT7
            theLight.Ambient = .Ambient
            theLight.Diffuse = .Diffuse
            theLight.Specular = .Specular
            theLight.dltType = .lType
            theLight.Position = .Position
            theLight.Range = .Range
            theLight.Attenuation0 = .Attenuation0
            theLight.Attenuation1 = .Attenuation1
            theLight.Attenuation2 = .Attenuation2
            theLight.Direction = .Direction
            theLight.Falloff = .Falloff
            theLight.Phi = .Phi
            theLight.Theta = .Theta
            .Enabled = TurnOn
            Device.SetLight .LightIndex, theLight
        End If
        Device.LightEnable .LightIndex, TurnOn
    End With
    Exit Sub
errs:
    Call anErr.AppErr(ErrString)
End Sub

Private Sub Class_Terminate()
    Set m_lights = Nothing
End Sub
